/*
Indoor Plant Care Assistant System

This sketch implements an IoT-based plant care system using an ESP32 microcontroller.
It monitors soil moisture and water levels near the plant, controls watering via a servo motor,
and manages LED lighting for optimal plant growth. The system interfaces with Blynk for 
user control and monitoring, and uses make.com for sending notifications via Telegram.

The circuit:
* Soil moisture sensor connected to pin 35
* Water level sensor connected to pin 34
* LED strip connected to pin 26
* Servo motor for watering connected to pin 33

The system operates in two modes:
1. Manual: User controls watering and lighting via Blynk app
2. Automatic: System autonomously manages plant care based on sensor readings

Video link: https://drive.google.com/file/d/1Olfpx7ve-IT3-vYND8_gT_AxlabPyn4j/view?usp=share_link

Created By:
Itay Flesh ID: 208295154
Noya Arav ID: 213103864

*/

// Blynk configuration
#define BLYNK_TEMPLATE_ID "" // insert your BLYNK_TEMPLATE_ID
#define BLYNK_TEMPLATE_NAME "" // insert your BLYNK_TEMPLATE_NAME
#define BLYNK_AUTH_TOKEN "" // insert your BLYNK_AUTH_TOKEN

// Include necessary libraries
#include <WiFi.h>
#include <WiFiClient.h>
#include <BlynkSimpleEsp32.h>
#include "DHTesp.h"
#include <FastLED.h>
#include <ESP32Servo.h>
#include <time.h>
#include <HTTPClient.h>

// WiFi credentials
char ssid[] = ""; // insert your wifi ssid
char pass[] = ""; // insert your wifi pass

// Pin definitions
const int soilMoisturePin = 35;
const int waterSensorPin = 34;
const int ledStripPin = 26;
const int servoPin = 33;

// LED strip configuration
#define NUM_LEDS 12
#define LED_TYPE WS2812
#define COLOR_ORDER GRB
CRGB leds[NUM_LEDS];

// Servo motor setup
Servo wateringServo;

// Timing variables
unsigned long previousMillis = 0;
const long interval = 5000;  // 5 seconds interval for sensor readings
unsigned long previousHourlyCheck = 0;
const long check_interval = 3600000;  // 1 hour in milliseconds

// State variables
bool ledState = false;
bool automaticMode = false;
int soilMoisturePercentage = 0;
int waterPercentage = 0;

// NTP server configuration
const char* ntpServer = "pool.ntp.org";
const long  gmtOffset_sec = 7200;  // Israel timezone offset in seconds
const int   daylightOffset_sec = 3600;

// Function to control LED strip
void setLEDState(bool state) {
  ledState = state;
  if (ledState) {
    if(soilMoisturePercentage < 40){
      fill_solid(leds, NUM_LEDS, CRGB::Red); 
    }
    else{
      fill_solid(leds, NUM_LEDS, CRGB::Yellow);  
    } 
  } else {
    fill_solid(leds, NUM_LEDS, CRGB::Black);
  }
  FastLED.show();
  Serial.println(ledState ? "LED Strip ON" : "LED Strip OFF");
}

// Function to water the plant
void waterPlant() {
  wateringServo.write(0);
  delay(3000);  // Water for 3 seconds
  wateringServo.write(180);
  Serial.println("Watering completed");
}

// Blynk virtual pin handlers
BLYNK_WRITE(V2) {  // Mode selection button
  automaticMode = param.asInt();
  Serial.println(automaticMode ? "Automatic Mode ON" : "Manual Mode ON");
}

BLYNK_WRITE(V3) {  // Manual watering button
  if (!automaticMode && param.asInt()) {
    waterPlant();
  }
}

BLYNK_WRITE(V5) {  // Manual LED control button
  if (!automaticMode) {
    setLEDState(param.asInt());
  }
}

void setup() {
  Serial.begin(115200);
  
  // Initialize LED strip
  FastLED.addLeds<LED_TYPE, ledStripPin, COLOR_ORDER>(leds, NUM_LEDS).setCorrection(TypicalLEDStrip);
  FastLED.setBrightness(50);
  
  // Initialize servo
  wateringServo.attach(servoPin);
  wateringServo.write(0);

  // Connect to WiFi
  WiFi.begin(ssid, pass);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("\nConnected to WiFi successfully!");

  // Configure time
  configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);

  // Connect to Blynk
  Blynk.begin(BLYNK_AUTH_TOKEN, ssid, pass);
  while (!Blynk.connected()) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("\nConnected to Blynk successfully!");
}

// Function to send notifications via Make.com
void sendNotification(String event, String message) {
  if(WiFi.status() == WL_CONNECTED) {
    HTTPClient http;
    http.begin("https://hook.eu2.make.com/7wg5fn4ksm2dd29orsna62n6b8txnsxd"); 
    http.addHeader("Content-Type", "application/json");
    
    String payload = "{\"event\":\"" + event + "\", \"message\":\"" + message + "\"}";
    
    int httpResponseCode = http.POST(payload);
    
    if(httpResponseCode > 0) {
      String response = http.getString();
      Serial.println(httpResponseCode);
      Serial.println(response);
    } else {
      Serial.print("Error on sending POST: ");
      Serial.println(httpResponseCode);
    }
    http.end();
  }
}



void loop() {
  Blynk.run();
  unsigned long currentMillis = millis();
  
  // Regular sensor readings (every 5 seconds)
  if (currentMillis - previousMillis >= interval) {
    previousMillis = currentMillis;
    
    // Read soil moisture
    int soilMoisture = analogRead(soilMoisturePin);
    soilMoisturePercentage = map(soilMoisture, 4095, 0, 0, 100);
    
    // Read water level
    int waterLevel = analogRead(waterSensorPin);
    waterPercentage = map(waterLevel, 0, 4095, 0, 100);
    
    // Update Blynk app
    Blynk.virtualWrite(V1, soilMoisturePercentage);
    Blynk.virtualWrite(V4, waterPercentage);
    
    // Log sensor data
    Serial.printf("Soil Moisture: %d%%, Water Level: %d%%, LED: %s, Mode: %s\n", 
                  soilMoisturePercentage, waterPercentage, ledState ? "ON" : "OFF",
                  automaticMode ? "Automatic" : "Manual");
                  
  }

  // Hourly check for needed automatic/manual mode actions
  if (currentMillis - previousHourlyCheck >= check_interval) {
    previousHourlyCheck = currentMillis;

    struct tm timeinfo;
    if(!getLocalTime(&timeinfo)){
      Serial.println("Failed to obtain time");
      return;
    }

    if (automaticMode) {
      // Automatic mode logic
      // Check if it's between 7 PM and 6 AM
      if (timeinfo.tm_hour >= 19 || timeinfo.tm_hour < 6) {
        setLEDState(true);
        sendNotification("Light Alert - Automatic Mode", "It's after sunset. The system has turned on the light for the plant.");
      } else {
        setLEDState(false);
      }

      // Check soil moisture
      if (soilMoisturePercentage < 40) {
        sendNotification("Soil moisture Alert - Automatic Mode", "Low soil moisture detected. The system has automatically turned on a warm-colored light to assist plant growth.");
        setLEDState(true);
      }
      // Check water level
      if (waterPercentage < 35) {
        sendNotification("Water Alert - Automatic Mode", "The plant needed to be watered. The system has watered it.");
        waterPlant();
      }
      
    } else {
      // Manual mode logic
      // Check if it's between 7 PM and 6 AM
      if (timeinfo.tm_hour >= 19 || timeinfo.tm_hour < 6) {
        sendNotification("Light Alert - Manual Mode", "It's after sunset. It is recommended to turn on the light for the plant.");
      } 

      // Check soil moisture
      if (soilMoisturePercentage < 40) {
        sendNotification("Soil moisture Alert - Manual Mode", "Low soil moisture detected. Please turn on a warm-colored light to assist plant growth.");
      }
      // Check water level
      if (waterPercentage < 35) {
        sendNotification("Water Alert - Manual Mode", "According to the plant's water level value, it is recommended to water the plant.");
      }
    }
  }
}
